# -*- coding: utf-8 -*-
# Author: Jimei Shen
import numpy as np
import cv2

ALLOWED_EXTENSIONS = set(['png', 'jpg', 'JPG', 'PNG', 'bmp', 'jpeg'])


verbose=True
showMainImg=True
BLOWUP_FACTOR = 1 # Resizes image before doing the algorithm. Changing to 2 makes things really slow. So nevermind on this.
RELEVANT_DIST_FOR_CORNER_GRADIENTS = 8*BLOWUP_FACTOR
dilationWidth = 1+2*BLOWUP_FACTOR #must be an odd number
dilationHeight = 1+2*BLOWUP_FACTOR #must be an odd number
dilationKernel = np.ones((dilationHeight,dilationWidth),'uint8')
writeEyeDebugImages = False #enable to export image files showing pupil center probability
eyeCounter = 0
lastCornerProb = np.ones([1,1])


# init the filters we'll use below
haarFaceCascade = cv2.CascadeClassifier("./eyecatching/haarcascades/haarcascade_frontalface_alt.xml")
haarEyeCascade = cv2.CascadeClassifier("./eyecatching/haarcascades/haarcascade_eye.xml")
OffsetRunningAvg = None
PupilSpacingRunningAvg = None

# global stuff for Adam's virtual ref point
#initialize the SURF descriptor
hessianThreshold = 500
nOctaves = 4
nOctaveLayers = 2
extended = True
upright = True
detector = cv2.xfeatures2d.SURF_create(hessianThreshold, nOctaves, nOctaveLayers, extended, upright)
#figure out a way to nearest neighbor map to index
virtualpoint = None
warm=0

useSURFReference = True
RANSAC_MIN_INLIERS = 7

WINDOW_NAME = "EyeTracking"
initTime = 10
plotNumber = 9
# pointPerPlot = 5
plotNColumn = 3

mode = 'test' #deploy


# -*- coding: utf-8 -*-
# Author: Jimei Shen
import cv2
import datetime
import pandas as pd
from eyecatching.eyeDetect import *
from eyecatching.settings import *
import pickle

class VideoCamera(object):
    def __init__(self):
        self.video = cv2.VideoCapture(0)

    def get_frame(self):

        run_time = 180+initTime
        start_time = datetime.datetime.now()
        res = pd.DataFrame(columns=['time', 'x', 'y'])

        if self.video.isOpened():  # try to get the first frame
            (readSuccessful, frame) = self.video.read()
        else:
            raise (Exception("failed to open camera."))
            readSuccessful = False

        while datetime.datetime.now()<= start_time+datetime.timedelta(seconds=run_time) and readSuccessful:
            pupilOffsetXYList = getOffset(frame, allowDebugDisplay=True)
            cv2.waitKey(10)
            print(datetime.datetime.now(), pupilOffsetXYList)
            if pupilOffsetXYList != None:
                res = res.append(pd.DataFrame(
                    {'time': [datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")], 'x': [pupilOffsetXYList[0]],
                     'y': [pupilOffsetXYList[1]]})).reset_index(drop=True)

            (readSuccessful, frame) = self.video.read()

        with open('pupilOffsetXYList.pkl', 'wb') as f:
            pickle.dump(res, f)
            f.close()

        ret, jpeg = cv2.imencode('.jpg', frame)

        return jpeg.tobytes()


#-------------------------------------------------------------------------------
# Name:        module1
# Purpose:
#
# Author:      lukea_000
#
# Created:     11/12/2013
# Copyright:   (c) lukea_000 2013
# Licence:     <your licence>
#-------------------------------------------------------------------------------

import numpy as np
import cv2
from eyecatching.eyeDetect import *

UPPER = 1000
UP_STEP = 1
DOWN_STEP = -1

class ClassyVirtualReferencePoint(object):
    class mykeypoint(object):
        def __init__(self, vector = (0,0), guess = (0, 0), found = 0, weight = 0):
            self.vector = vector
            self.guess = guess
            self.found = found
            self.weight = weight

    """
        Tracks a batch of keypoints, and outputs how they vote on a virtual
        reference point in an image.

        eventually will pick up new keypoints as it goes, but doesn't yet.

        by convention, all vote vectors will be of the form (referencepoint - a)
    """
    def __init__(self, keypoints, descriptors, referencePointYX, bounds, eye1, eye2):
        """
            takes keypoints and descriptors for an image, and sets up the
            internals of the class as though it's just been run.
            internals:
                knn classifier
                reference point

            the main array of descriptors is also set up, with:
                 descriptors
                 vector to virtual point
                 matchedLocation: (None if not matched)
                 weight
            as representation may change, setters and gettters should be used
        """
        self.reference = referencePointYX
        assert(len(descriptors) > 0)
        self.rowsize = len(descriptors) / len(keypoints)
        #this doesn't need to be self.saved here now, but I may want it later:
        keypoints, self.descriptors = self.cropToBounds(keypoints, descriptors, bounds, eye1, eye2)
        labels = np.arange(len(keypoints), dtype = np.int32)
        self.oldknn = cv2.ml.KNearest_create()
        #train the KNN matcher
        self.oldknn.train(self.descriptors,cv2.ml.ROW_SAMPLE, labels)
        self.keypointdata = []
        for kp in keypoints:
            vector = (self.reference[0] - kp.pt[0], self.reference[1] - kp.pt[1])
            data = self.mykeypoint(vector = vector)
            self.keypointdata.append(data)

    def drawPt(self, cx, cy, output):
        color = (255, 0, 255, 100)
        cx, cy = int(cx), int(cy)
        cv2.circle(output, (cx, cy), 5, color, thickness=2)

    def calculateReferencePoint(self):
        """
            does math for average x & y
        """
        weightedx, weightedy = (0, 0)
        denom = 0
        for kp in self.keypointdata:
            weightedx += kp.found * kp.weight * kp.guess[0]
            weightedy += kp.found * kp.weight * kp.guess[1]
            denom += kp.weight
        weightedx = weightedx / denom
        weightedy = weightedy / denom
        #set the reference point, maybe do some smoothing later
        self.reference = weightedx, weightedy

        #zero out the others if your way of setting weights changes
        for i, kp in enumerate(self.keypointdata):
            self.keypointdata[i].found = 0
        #return this
        return self.reference

    def cropToBounds(self, kps, des, bounds, eye1, eye2):
#        des = np.array(des, dtype = np.float32).reshape((-1, self.rowsize))
        keypoints, descriptors = [], []
        for i, kp in enumerate(kps):
            if containsPoint(bounds, kp.pt): #point is in the face
                if not containsPoint(eye1, kp.pt) and not containsPoint(eye2, kp.pt):
                #point is outside both eyes
                    keypoints.append(kp)
                    descriptors.append(des[i])
#        return (keypoints, np.array(descriptors, dtype = np.float32).reshape((-1, self.rowsize)))
        return (keypoints,np.array(descriptors))

    def getReferencePoint(self, keypoints, descriptors, bounds, eye1, eye2, img = None):
        """
            takes in new keypoints, and descriptors
            finds the two closest matches for each incoming one, and then verifies that they are close enough
            using D. Lowe's ratio test

            for each close enough point, it sets the weight equal to one
        """
        keypoints, descriptors = self.cropToBounds(keypoints, descriptors, bounds, eye1, eye2)
        _, _, oldneighborLabels, olddistances = self.oldknn.findNearest(descriptors, 2)
        newknn = cv2.ml.KNearest_create()
        newlabels = np.arange(len(keypoints), dtype=np.float32)
        newknn.train(descriptors, cv2.ml.ROW_SAMPLE, newlabels)
        _, _, newneighborLabels, newdistances = newknn.findNearest(self.descriptors, 2)
        #import pdb;
        for i, dists in enumerate(olddistances):
            closestDistance, other = dists
            if closestDistance < other * .7:
                #its a close match, but is it a bijection?
                #want to know whether the label of the old neighbor was matched to this new guy too
                #I know about what I have:
                #old label is the label of the nearest old point.
                oldlabel = int(oldneighborLabels[i][0])
                #want to know whether the old point at index oldlabel was labeled i, because if so, then its bijective
                if int(newneighborLabels[oldlabel][0]) ==  i:
                    newdist = newdistances[oldlabel]
                    closestDistance, other = newdist
                    if closestDistance < other * .7:
                        #really a match
                        #what index is it at?
                        self.keypointdata[oldlabel].found = 1
                        self.keypointdata[oldlabel].weight = min(UPPER, self.keypointdata[oldlabel].weight + UP_STEP)
                        #pdb.set_trace()
                        self.keypointdata[oldlabel].guess = keypoints[i].pt[0] + self.keypointdata[oldlabel].vector[0], \
                                                 keypoints[i].pt[1] + self.keypointdata[oldlabel].vector[1]
                        #if there's an image, draw on it
                        if img is not None:
                            cx, cy = keypoints[i].pt#this is different from my other code but I think it's good
                            cx, cy = int(cx), int(cy)
                            cv2.putText(img, str(oldlabel), (cx, cy), cv2.FONT_HERSHEY_SIMPLEX, .25, (100, 170, 0))
        #step down the weights of the not found ones
        for kp in self.keypointdata:
            if kp.found == 0:
                kp.weight = 0
        if max([kp.weight for kp in self.keypointdata]) > 0: #if we've found at least 1 keypoint, recalculate the virtual reference
            self.calculateReferencePoint()
        if img is not None:
            self.drawPt(self.reference[0], self.reference[1], img)
        return (self.reference)


def containsPoint(outerFeature, p):
    #eyes are arrays of the form [minX, minY, maxX, maxY]
    return p[0] > outerFeature[0] and p[0] < outerFeature[2] and p[1] > outerFeature[1] and p[1] < outerFeature[3]


# -*- coding: utf-8 -*-

from flask import Flask, render_template, Response, request, jsonify
import os
from eyecatching.readRes import *
from eyecatching.settings import *
import time
from eyecatching.camera import VideoCamera
from werkzeug.utils import secure_filename

app = Flask(__name__)

video_stream = VideoCamera()

def gen(camera):
    frame = camera.get_frame()
    yield (b'--frame\r\n'
               b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n\r\n')

@app.route('/video_feed')
def video_feed():
     return Response(gen(video_stream),
                    mimetype='multipart/x-mixed-replace; boundary=frame')


def allowed_file(filename):
    return '.' in filename and filename.rsplit('.', 1)[1] in ALLOWED_EXTENSIONS

@app.route('/', methods=['POST', 'GET'])  # 添加路由
def upload():
    if request.method == 'POST':
        f = request.files['file']

        if not (f and allowed_file(f.filename)):
            return jsonify({"error": 1001, "msg": "Only for: png / PNG / jpg / JPG/ bmp / jpeg"})

        global user_input
        user_input = request.form.get("name")

        global pointPerPlot
        pointPerPlot = request.form.get("plot_points")

        basepath = os.path.dirname(__file__)

        upload_path = os.path.join(basepath, 'static', secure_filename(f.filename))
        f.save(upload_path)

        img = cv2.imread(upload_path)
        cv2.imwrite(os.path.join(basepath, 'static', user_input), img)

        return render_template('upload_ok.html', userinput=user_input, val1=time.time())

    return render_template('upload.html')


@app.route('/plot/', methods=['POST', 'GET'])
def resPlotAll():
    plot_url = plot(user_input, pointPerPlot)
    return render_template('plot_track.html', plot_url = plot_url);



from eyecatching.settings import *
import math
from eyecatching.ClassyVirtualReferencePoint import ClassyVirtualReferencePoint

# set doTraining = False to display debug graphics:
# You should do this first. There should be a green line from your
# forehead to one pupil; the end of the line is the estimate of pupil position. The blue
# circles should generally track your pupils, though less reliably than the green line.
# If performance is bad, you can tweak the "TUNABLE PARAMETER" lines. (This is a big
# area where improvement is needed; probably some learning of parameters.)
# Set True to run the main program:
# You click where you're looking, and, after around 10-20 such clicks,
# the program will learn the correspondence and start drawing a blue blur
# where you look. It's important to keep your head still (in position AND angle)
# or it won't work.


def featureCenter(f):
    return (.5*(f.mExtents[0]+f.mExtents[1]),.5*(f.mExtents[2]+f.mExtents[3]) )

# returns center in form (y,x)
def featureCenterXY(rect):
    #eyes are arrays of the form [minX, minY, maxX, maxY]
    return (.5*(rect[0]+rect[2]), .5*(rect[1]+rect[3]))

def centeredBox(feature1, feature2, boxWidth, boxHeight, yOffsetToAdd = 0):
    f1 = np.array(featureCenterXY(feature1))
    f2 = np.array(featureCenterXY(feature2))
    center = (f1[:]+f2[:])/2
    center[1] += yOffsetToAdd
    offset = np.array([boxWidth/2,boxHeight/2])
    return np.concatenate( (center-offset, center+offset) )


def contains(outerFeature, innerFeature):
    p = featureCenterXY(innerFeature)
    #eyes are arrays of the form [minX, minY, maxX, maxY]
    return p[0] > outerFeature[0] and p[0] < outerFeature[2] and p[1] > outerFeature[1] and p[1] < outerFeature[3]

def containsPoint(outerFeature, p):
    #eyes are arrays of the form [minX, minY, maxX, maxY]
    return p[0] > outerFeature[0] and p[0] < outerFeature[2] and p[1] > outerFeature[1] and p[1] < outerFeature[3]

# Takes an ndarray of face rects, and an ndarray of eye rects.
# Returns the first eyes that are inside the face but not inside each other.
# Eyes are returned as the tuple (leftEye, rightEye)
def getLeftAndRightEyes(faces, eyes):
    #loop through detected faces. We'll do our processing on the first valid one.
    if len(eyes)==0:
        return ()
    for face in faces:
        for i in range(eyes.shape[0]):
            for j in range(i+1,eyes.shape[0]):
                leftEye = eyes[i] #by left I mean camera left
                rightEye = eyes[j]
                #eyes are arrays of the form [minX, minY, maxX, maxY]
                if (leftEye[0]+leftEye[2]) > (rightEye[0]+rightEye[2]): #leftCenter is > rightCenter
                    rightEye, leftEye = leftEye, rightEye #swap
                if contains(leftEye,rightEye) or contains(rightEye, leftEye):#they overlap. One eye containing another is due to a double detection; ignore it
                    debugPrint('rejecting double eye')
                    continue
                if leftEye[3] < rightEye[1] or rightEye[3] < leftEye[1]:#top of one is below (>) bottom of the other. One is likely a mouth or something, not an eye.
                    debugPrint('rejecting non-level eyes')
                    continue
##                if leftEye.minY()>face.coordinates()[1] or rightEye.minY()>face.coordinates()[1]: #top of eyes in top 1/2 of face
##                    continue;
                if not (contains(face,leftEye) and contains(face,rightEye)):#face contains the eyes. This is our standard of humanity, so capture the face.
                    debugPrint("face doesn't contain both eyes")
                    continue
                return (leftEye, rightEye)

    return ()


def debugPrint(s):
    if verbose:
        print(s)


# Returns (cy,cx) of the pupil center, where y is down and x is right. You should pass in a grayscale Cv2 image which
# is closely cropped around the center of the eye (using the Haar cascade eye detector)
def getPupilCenter(gray, getRawProbabilityImage=False):
##    (scleraY, scleraX) = np.unravel_index(gray.argmax(),gray.shape)
##    scleraColor = colors[scleraY,scleraX,:]
##    img[scleraX,scleraY] = (255,0,0)
##    img.colorDistance(skinColor[:]).save(disp)
##    img.edges().save(disp)
##    print skinColor, scleraColor
    gray = gray.astype('float32')
    if BLOWUP_FACTOR != 1:
        gray = cv2.resize(gray, (0,0), fx=BLOWUP_FACTOR, fy=BLOWUP_FACTOR, interpolation=cv2.INTER_LINEAR)

    IRIS_RADIUS = gray.shape[0]*.75/2 #conservative-large estimate of iris radius TODO: make this a tracked parameter--pass a prior-probability of radius based on last few iris detections. TUNABLE PARAMETER
    #debugImg(gray)
    dxn = cv2.Sobel(gray,cv2.CV_32F,1,0,ksize=3) #optimization opportunity: blur the image once, then just subtract 2 pixels in x and 2 in y. Should be equivalent.
    dyn = cv2.Sobel(gray,cv2.CV_32F,0,1,ksize=3)
    magnitudeSquared = np.square(dxn)+np.square(dyn)

    # ########### Pupil finding
    magThreshold = magnitudeSquared.mean()*.6 #only retain high-magnitude gradients. <-- VITAL TUNABLE PARAMETER
                    # The value of this threshold is critical for good performance.
                    # todo: adjust this threshold using more images. Maybe should train our tuned parameters.
    # form a bool array, unrolled columnwise, which can index into the image.
    # we will only use gradients whose magnitude is above the threshold, and
    # (optionally) where the gradient direction meets characteristics such as being more horizontal than vertical.
    gradsTouse = (magnitudeSquared>magThreshold) & (np.abs(4*dxn)>np.abs(dyn))
    lengths = np.sqrt(magnitudeSquared[gradsTouse]) #this converts us to double format
    gradDX = np.divide(dxn[gradsTouse],lengths) #unrolled columnwise
    gradDY = np.divide(dyn[gradsTouse],lengths)

    isDark = gray< (gray.mean()*.8)  #<-- TUNABLE PARAMETER
    global dilationKernel
    isDark = cv2.dilate(isDark.astype('uint8'), dilationKernel) #dilate so reflection goes dark too

    gradXcoords =np.tile( np.arange(dxn.shape[1]), [dxn.shape[0], 1])[gradsTouse] # build arrays holding the original x,y position of each gradient in the list.
    gradYcoords =np.tile( np.arange(dxn.shape[0]), [dxn.shape[1], 1]).T[gradsTouse] # These lines are probably an optimization target for later.
    minXForPupil = 0 #int(dxn.shape[1]*.3)

    centers = np.array([[phiWithHist(cx,cy,gradDX,gradDY,gradXcoords,gradYcoords, IRIS_RADIUS) if isDark[cy][cx] else 0 for cx in range(minXForPupil,dxn.shape[1])] for cy in range(dxn.shape[0])]).astype('float32')

    maxInd = centers.argmax()
    (pupilCy,pupilCx) = np.unravel_index(maxInd, centers.shape)
    pupilCx += minXForPupil
    pupilCy /= BLOWUP_FACTOR
    pupilCx /= BLOWUP_FACTOR
    if writeEyeDebugImages:
        global eyeCounter
        eyeCounter = (eyeCounter+1)%5 #write debug image every 5th frame
        if eyeCounter == 1:
            cv2.imwrite( "eyeGray.png", gray/gray.max()*255) #write probability images for our report
            cv2.imwrite( "eyeIsDark.png", isDark*255)
            cv2.imwrite( "eyeCenters.png", centers/centers.max()*255)
    if getRawProbabilityImage:
        return (pupilCy, pupilCx, centers)
    else:
        return (pupilCy, pupilCx)


#This was a failed attempt to find eye corners, not used in final version.
# Returns (cy,cx) of the eye corner, where y is down and x is right. You should pass in a grayscale Cv2 image which
# is closely cropped around the corner of the eye (using the Haar cascade eye detector)
def getEyeCorner(gray):
##    (scleraY, scleraX) = np.unravel_index(gray.argmax(),gray.shape)
##    scleraColor = colors[scleraY,scleraX,:]
##    img[scleraX,scleraY] = (255,0,0)
##    img.colorDistance(skinColor[:]).save(disp)
##    img.edges().save(disp)
##    print skinColor, scleraColor

    if BLOWUP_FACTOR != 1:
        gray = cv2.resize(gray, (0,0), fx=BLOWUP_FACTOR, fy=BLOWUP_FACTOR, interpolation=cv2.INTER_LINEAR)
    gray = gray.astype('float32')

    dxn = cv2.Sobel(gray,cv2.CV_32F,1,0,ksize=3) #optimization opportunity: blur the image once, then just subtract 2 pixels in x and 2 in y. Should be equivalent.
    dyn = cv2.Sobel(gray,cv2.CV_32F,0,1,ksize=3)
    magnitudeSquared = np.square(dxn)+np.square(dyn)

    rangeOfXForCorner = int(dxn.shape[1]/2)
    magThreshold = magnitudeSquared.mean()*.5 #only retain high-magnitude gradients. todo: adjust this threshold using more images. Maybe should train our tuned parameters.
    # form a bool array, unrolled columnwise, which can index into the image.
    # we will only use gradients whose magnitude is above the threshold, and
    # (optionally) where the gradient direction meets characteristics such as being more horizontal than vertical.
    gradsTouse = (magnitudeSquared>magThreshold) & (np.abs(2*dyn)>np.abs(dxn))
    lengths = np.sqrt(magnitudeSquared[gradsTouse])
    gradDX = np.divide(dxn[gradsTouse],lengths) #unrolled columnwise
    gradDY = np.divide(dyn[gradsTouse],lengths)
    gradXcoords =np.tile( np.arange(dxn.shape[1]), [dxn.shape[0], 1])[gradsTouse] # build arrays holding the original x,y position of each gradient in the list.
    gradYcoords =np.tile( np.arange(dxn.shape[0]), [dxn.shape[1], 1]).T[gradsTouse] # These lines are probably an optimization target for later.

    centers = np.array([[phiCorner(cx,cy,gradDX,gradDY,gradXcoords,gradYcoords) for cx in range(rangeOfXForCorner)] for cy in range(dxn.shape[0])])
    global lastCornerProb
    weightOnNew = 1
    prior = np.ones(centers.shape)*(lastCornerProb.mean()*.5)*(1-weightOnNew) # fill with default value
    startPrior = [0,0]
    endPrior = [0,0]
    startNew = [0,0]
    endNew = [0,0]
    for i in range(2):
        diff = lastCornerProb.shape[i]-centers.shape[i]
        if diff >= 0: # new is smaller
            startPrior[i] = int(diff/2)
            endPrior[i] = startPrior[i]+centers.shape[i]
            startNew[i]=0
            endNew[i]=centers.shape[i]
        else: # prior is smaller
            startPrior[i] = 0
            endPrior[i] = lastCornerProb.shape[i]
            startNew[i]=int(-diff/2)
            endNew[i]=startNew[i]+lastCornerProb.shape[i]
    prior[startNew[0]:endNew[0]][startNew[1]:endNew[1]] = (1-weightOnNew)*lastCornerProb[startPrior[0]:endPrior[0]][startPrior[1]:endPrior[1]]
    centers = centers * (1/centers.max())
    centers = centers * (weightOnNew + prior)
    (cornerCy,cornerCx) = np.unravel_index(centers.argmax(), centers.shape)
    cornerCy /= BLOWUP_FACTOR
    cornerCx /= BLOWUP_FACTOR
    return (cornerCy, cornerCx)

# Estimates the probability that the given cx,cy is the pupil center, by taking
# (its vector to each gradient location) dot (the gradient vector)
def phi(cx,cy,gradDX,gradDY,gradXcoords,gradYcoords, IRIS_RADIUS):
    vecx = gradXcoords-cx
    vecy = gradYcoords-cy
    lengthsSquared = np.square(vecx)+np.square(vecy)
    valid = (lengthsSquared > 0) & (lengthsSquared < IRIS_RADIUS**2) #avoid divide by zero, only use nearby gradients.
    dotProd = np.multiply(vecx,gradDX)+np.multiply(vecy,gradDY)
    valid = valid & (dotProd > 0) # only use vectors in the same direction (i.e. the dark-to-light transition direction is away from us. The good gradients look like that.)
    dotProd = np.square(dotProd[valid]) # dot products squared
    dotProd = np.divide(dotProd,lengthsSquared[valid]) #normalized squared dot products. Should range from 0 to 1.
    dotProd = dotProd[dotProd > .9] #only count dot products that are really close
    return np.sum(dotProd) # this is equivalent to normalizing vecx and vecy, because it takes dotProduct^2 / length^2

# Estimates the probability that the given cx,cy is the pupil center, by taking
# (its vector to each gradient location) dot (the gradient vector)
# only uses gradients which are near the peak of a histogram of distance
# cx and cy may be integers or floating point.
def phiWithHist(cx,cy,gradDX,gradDY,gradXcoords,gradYcoords, IRIS_RADIUS):
    vecx = gradXcoords-cx
    vecy = gradYcoords-cy
    lengthsSquared = np.square(vecx)+np.square(vecy)
    # bin the distances between 1 and IRIS_RADIUS. We'll discard all others.
    binWidth = 1 #TODO: account for webcam resolution. Also, maybe have it transform ellipses to circles when on the sides? (hard)
    numBins =  int(np.ceil((IRIS_RADIUS-1)/binWidth))
    bins = [(1+binWidth*index)**2 for index in range(numBins+1)] #express bin edges in terms of length squared
    hist = np.histogram(lengthsSquared, bins)[0]
    maxBin = hist.argmax()
    slop = binWidth
    valid = (lengthsSquared > max(1,bins[maxBin]-slop)) &  (lengthsSquared < bins[maxBin+1]+slop) #use only points near the histogram distance
    dotProd = np.multiply(vecx,gradDX)+np.multiply(vecy,gradDY)
    valid = valid & (dotProd > 0) # only use vectors in the same direction (i.e. the dark-to-light transition direction is away from us. The good gradients look like that.)
    dotProd = np.square(dotProd[valid]) # dot products squared
    dotProd = np.divide(dotProd,lengthsSquared[valid]) #make normalized squared dot products
    dotProd = np.square(dotProd) # squaring puts an even higher weight on values close to 1
    return np.sum(dotProd) # this is equivalent to normalizing vecx and vecy, because it takes dotProduct^2 / length^2

# Failed attempt to find probability that the given cx,cy is an eye corner, not
# used in final version. Works by taking
# (its vector to each gradient location) dot (the gradient vector). The corner
# should have a near-zero dot product with its nearby vectors, because the
# eyelids form lines that point right at the eye corner (so their gradients are
# at a 90 degree angle to it).
# only uses gradients which are near the peak of a histogram of distance
# cx and cy may be integers or floating point.
def phiCorner(cx,cy,gradDX,gradDY,gradXcoords,gradYcoords):
    vecx = gradXcoords-cx
    vecy = gradYcoords-cy
    angles = np.arctan2(vecy,vecx)
    lengthsSquared = np.square(vecx)+np.square(vecy)
    valid = (lengthsSquared > 0) & (lengthsSquared < RELEVANT_DIST_FOR_CORNER_GRADIENTS) & (vecx>0.4)#RIGHT EYE ASSUMPTION
    numBins = 10
    (hist,bins) = np.histogram(angles, numBins, (-math.pi,math.pi))
    slop = math.pi/numBins/2
    maxBin = hist.argmax()
    hist[maxBin] = 0;
    hist[max(0,maxBin-1)]=0;
    hist[min(maxBin+1,numBins-1)]=0;
    secondMaxBin = hist.argmax();
    stat = angles #gradDY
    validBina = valid & ((bins[maxBin]-slop<stat)&(stat<bins[maxBin+1]+slop))
    validBinb = valid & ((bins[secondMaxBin]-slop<stat)&(stat<bins[secondMaxBin+1]+slop))#use only points near the histogram max


    dotProd = np.multiply(vecx,gradDX)+np.multiply(vecy,gradDY)
    dotProda = np.square(dotProd[validBina]) # dot products squared
    dotProdb = np.square(dotProd[validBinb]) # dot products squared
    dotProda = 1.0-np.divide(dotProda,lengthsSquared[validBina]) #make normalized squared dot products, and take 1-them so 0 gets the highest score
    dotProdb = 1.0-np.divide(dotProdb,lengthsSquared[validBinb]) #make normalized squared dot products, and take 1-them so 0 gets the highest score
    dotProda = np.square(dotProda) #only count dot products that are really close
    dotProdb = np.square(dotProdb) #only count dot products that are really close
    suma = np.sum(dotProda) # this is equivalent to normalizing vecx and vecy, because it takes dotProduct^2 / length^2
    sumb = np.sum(dotProdb) # this is equivalent to normalizing vecx and vecy, because it takes dotProduct^2 / length^2
    return min(suma,sumb)+.5*max(suma,sumb) #this score should favor a strong bimodal histogram shape

#for debugging
def phiTest(cx,cy,gradDX,gradDY,gradXcoords,gradYcoords):
    for ix,xcoord in enumerate(gradXcoords):
        if xcoord==cx and gradYcoords[ix]==cy:
            return np.atan2(gradDY[ix],gradDX[ix])
    return 0

# multiplies newProb and priorToMultiply
# YXoffsetOfSecondWithinFirst - priorToMultiply will be shifted by this amount in space
# defaultPriorValue - if not all of newProb is covered by priorToMultiply, this scalar goes in the uncovered areas.
def multiplyProbImages(newProb, priorToMultiply, YXoffsetOfSecondWithinFirst, defaultPriorValue):
    if np.any(YXoffsetOfSecondWithinFirst > newProb.shape) or np.any(-YXoffsetOfSecondWithinFirst > priorToMultiply.shape):
        print("multiplyProbImages aborting - zero overlap. Offset and matrices:")
        print(YXoffsetOfSecondWithinFirst)
        print(newProb.shape)
        print(priorToMultiply.shape)
        return newProb*defaultPriorValue
    prior = np.ones(newProb.shape)*defaultPriorValue # Most of this will get overwritten. For areas that won't be, with fill with default value.
    #offsets
    startPrior = [0,0]
    endPrior = [0,0]
    startNew = [0,0]
    endNew = [0,0]
    for i in range(2):
        #offset=0
        # NOT THIS: x[1:2][1:2]
        # THIS: x[1:2,1:2]
        offset = int(round(YXoffsetOfSecondWithinFirst[i])) # how much to offset 'prior' within 'newProb', for the current dimension
        # print(offset)
        if offset >= 0: # prior goes right of 'newProb', in the world. So prior will be copied into newProb at a positive offset
            startPrior[i] = 0 #index within prior
            endPrior[i] = min(priorToMultiply.shape[i],newProb.shape[i]-offset) #how much of prior to copy
            startNew[i]=offset
            endNew[i]=offset+endPrior[i]
        else: # prior goes left of 'newProb', in the world.
            startPrior[i] = -offset
            endPrior[i] = min(priorToMultiply.shape[i], startPrior[i]+newProb.shape[i])
            startNew[i]=0
            endNew[i]=endPrior[i]-startPrior[i]
    prior[startNew[0]:endNew[0],startNew[1]:endNew[1]] = priorToMultiply[startPrior[0]:endPrior[0],startPrior[1]:endPrior[1]]
    #prior[1:10,1:10] = priorToMultiply[1:10,1:10]
    #now, prior holds the portion of priorToMultiply which overlapped newProb.
    return newProb * prior


## img: cv2 image in uint8 format
## cascade: object you made with cv2.CascadeClassifier("./haarcascades/haarcascade_frontalface_alt.xml")
## minimumFeatureSize (ySize,xSize) tuple holding the smallest object you'd be looking for. E.g. (30,30)
## returns a numpy ndarray where rects[0] is the first detection, and holds [minX, minY, maxX, maxY] where +Y = downward
def detect(img, cascade, minimumFeatureSize=(20,20)):
    if cascade.empty():
        raise(Exception("There was a problem loading your Haar Cascade xml file."))
    rects = cascade.detectMultiScale(img, scaleFactor=1.3, minNeighbors=1, minSize=minimumFeatureSize)
    if len(rects) == 0:
        return []
    rects[:,2:] += rects[:,:2] #convert last coord from (width,height) to (maxX, maxY)
    return rects

def draw_rects(img, rects, color):
    for x1, y1, x2, y2 in rects:
        cv2.rectangle(img, (x1, y1), (x2, y2), color, 2)



#*********  getOffset  **********
# INPUTS:
# frame - a color numpy image.
# allowDebugDisplay - pass True if you want it to draw pupil centers, etc on "frame" and then display it.
# Display requires that you called this line to create the window: previewWindow = cv2.namedWindow(WINDOW_NAME)
# trackAverageOffset - output will be a moving average rather than instantaneous value
# directInferenceLeftRight - combines probability images from left and right to hopefully reduce noise in estimation of pupil offset
# Returns a list of two tuples of pupil offsets from the forehead dot. Specifically:
# [(cameraLeftEyeOffsetX, cameraLeftEyeOffsetY),  (cameraRightEyeOffsetX, cameraRightEyeOffsetY) ]
# If no valid face is found, returns None.
# Requires the functions above.

def getOffset(frame, allowDebugDisplay=True, trackAverageOffset=True, directInferenceLeftRight=True):
    gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
    gray = cv2.equalizeHist(gray)
    # find faces and eyes
    minFaceSize = (80,80)
    minEyeSize = (25,25)
    faces = detect(gray,haarFaceCascade,minFaceSize)
    eyes = detect(gray,haarEyeCascade,minEyeSize)
    drawKeypoints = allowDebugDisplay #can set this false if you don't want the keypoint ID numbers
    if allowDebugDisplay:
        output = frame
        draw_rects(output,faces,(0,255,0)) #BGR format
    else:
        output = None
    leftEye_rightEye = getLeftAndRightEyes( faces, eyes)
    if leftEye_rightEye: #if we found valid eyes in a face
        xDistBetweenEyes = (leftEye_rightEye[0][0]+leftEye_rightEye[0][1]+leftEye_rightEye[1][0]+leftEye_rightEye[1][1])/4 #for debugging reference point
        pupilXYList = []
        pupilCenterEstimates = []
        for eyeIndex, eye in enumerate(leftEye_rightEye):
            corner = eye.copy()

            #eyes are arrays of the form [minX, minY, maxX, maxY]
            eyeWidth = eye[2]-eye[0]
            eyeHeight = eye[3]-eye[1]
            eye[0] += eyeWidth*.20
            eye[2] -= eyeWidth*.15
            eye[1] += eyeHeight*.3
            eye[3] -= eyeHeight*.2
            eye = np.round(eye)
            eyeImg = gray[eye[1]:eye[3], eye[0]:eye[2]]
            if directInferenceLeftRight:
                (cy,cx, centerProb) = getPupilCenter(eyeImg, True)
                pupilCenterEstimates.append(centerProb.copy())
            else:
                (cy,cx) = getPupilCenter(eyeImg, True)
            pupilXYList.append( (int(cx+eye[0]),int(cy+eye[1]))  )
            if allowDebugDisplay:
                cv2.rectangle(output, (eye[0], eye[1]), (eye[2], eye[3]), (0,255,0), 1)
                cv2.circle(output, pupilXYList[eyeIndex], 3, (255,0,0),thickness=1) #BGR format

        # direct inference combination of the two eye probability images.
        global PupilSpacingRunningAvg
        if directInferenceLeftRight:
            # these vectors are in XY format
            pupilSpacing = np.array(pupilXYList[1])-np.array(pupilXYList[0]) # vector from pupil 0 to pupil 1
            if PupilSpacingRunningAvg is None:
                PupilSpacingRunningAvg = pupilSpacing
            else:
                weightOnNew = .03
                PupilSpacingRunningAvg = (1-weightOnNew)*PupilSpacingRunningAvg + weightOnNew*pupilSpacing  # vector from pupil 0 to pupil 1
            if allowDebugDisplay:
                cv2.line(output, (int(pupilXYList[0][0]),int(pupilXYList[0][1])), (int(pupilXYList[0][0]+PupilSpacingRunningAvg[0]), int(pupilXYList[0][1]+PupilSpacingRunningAvg[1])), (0,100,100))
            imageZeroToOneVector = leftEye_rightEye[1][0:2]-leftEye_rightEye[0][0:2] # vector from eyeImg 0 to 1
            positionOfZeroWithinOne = PupilSpacingRunningAvg-imageZeroToOneVector; # the extra distance that wasn't covered by the bounding boxes should be applied as an offset when multiplying images.
            ksize = 5 #kernel size = x width and y height of the filter
            sigma = 2
            for i,centerEstimate in enumerate(pupilCenterEstimates):
                pupilCenterEstimates[i] = cv2.GaussianBlur(pupilCenterEstimates[i], (ksize,ksize), sigma, borderType=cv2.BORDER_REPLICATE)
            jointPupilProb = multiplyProbImages(pupilCenterEstimates[1], pupilCenterEstimates[0], positionOfZeroWithinOne[::-1], 0) # the [::-1] reverse the order, so it's YX instead of the XY that these vectors are in
            maxInd = jointPupilProb.argmax()
            (pupilCy,pupilCx) = np.unravel_index(maxInd, jointPupilProb.shape) # coordinates in the eye 1 (camera-right eye) image
            pupilXYList[0]=pupilXYList[1]=(pupilCx + leftEye_rightEye[1][0],pupilCy + leftEye_rightEye[1][1]) #convert to absolute image coordinates


        if not useSURFReference: # this code assumes you have drawn a dark dot on your forehead. Should be drawn between the eyes, about the size of the iris.
            dotSearchBox = np.round( centeredBox(leftEye_rightEye[0], leftEye_rightEye[1], xDistBetweenEyes*.2, xDistBetweenEyes*.3, -xDistBetweenEyes*.09 ) ).astype('int')

            (refY,refX) = getPupilCenter(gray[dotSearchBox[1]:dotSearchBox[3], dotSearchBox[0]:dotSearchBox[2]])
            refXY = (refX+dotSearchBox[0],refY+dotSearchBox[1])
            if allowDebugDisplay:
                cv2.rectangle(output, (dotSearchBox[0], dotSearchBox[1]), (dotSearchBox[2], dotSearchBox[3]), (128,0,128), 1)
                cv2.circle(output, refXY, 2, (0,0,100),thickness=1) #BGR format
        else: # Adam's virtual reference point code. See paper for how it works.
            refXY = (0,0)
            global warm, virtualpoint
            warm += 1
            if warm > 8:
                #adam
                face = faces[0]#expect the first one
                faceImg = gray[face[1]:face[3], face[0]:face[2]]
                cornerImg = gray[corner[1]:corner[3], corner[0]:corner[2]]
                if virtualpoint == None: #we haven't set up the reference point yet
                    haystackKeypoints, haystackDescriptors = detector.detectAndCompute(gray, mask=None)
                    if len(haystackKeypoints) != 0:
                        betweenEyes = (np.array(featureCenterXY(leftEye_rightEye[0]))+np.array(featureCenterXY(leftEye_rightEye[1])))/2
                        virtualpoint = ClassyVirtualReferencePoint(haystackKeypoints, haystackDescriptors, (betweenEyes[0], betweenEyes[1]), face, leftEye_rightEye[0], leftEye_rightEye[1])
                    else:
                        print("begin fail")
                else: #we've already created it
                    keypoints, descriptors = detector.detectAndCompute(gray, mask=None)
                    if drawKeypoints:
                        imgToDrawOn = output
                    else:
                        imgToDrawOn = None
                    if len(descriptors) != 0:
                        refXY  = virtualpoint.getReferencePoint(keypoints, descriptors, face, leftEye_rightEye[0], leftEye_rightEye[1], imgToDrawOn)
            # end of Adam's reference point code

        for i in range(len(pupilXYList)):
            pupilXYList[i] = ( pupilXYList[i][0]-refXY[0], pupilXYList[i][1]-refXY[1])
        pupilXYList = list(pupilXYList[0])+ list(pupilXYList[1]) #concatenate cam-left and cam-right coordinate tuples to make a single length 4 vector [x,y,x,y]

        if trackAverageOffset: # this frame's estimated offset will be a weighted average of the new measurement and the last frame's estimated offset
            global OffsetRunningAvg
            if OffsetRunningAvg is None:
                OffsetRunningAvg = np.array( [0,0])
            weightOnNew = .4; #Tuned parameter, must be >0 and <=1.0. Increase for faster response, decrease for better noise rejection.
            currentOffset = (np.array(pupilXYList[:2])+np.array(pupilXYList[2:]))/2
            OffsetRunningAvg = (1.0-weightOnNew)*OffsetRunningAvg + weightOnNew*currentOffset
            pupilXYList = OffsetRunningAvg
            if allowDebugDisplay:
                cv2.line(output, (int(refXY[0]),int(refXY[1])), (int(refXY[0]+pupilXYList[0]), int(refXY[1]+pupilXYList[1])), (0,255,100))

        return tuple(pupilXYList) # if trackAverageOffset, it's length 2 and holds the average offset. Else, it's length 4 (old code)



# -*- coding: utf-8 -*-
# Author: Jimei Shen

import datetime
import matplotlib.pyplot as plt
import math
from math import pi
import pandas as pd
import pickle
from random import sample
from eyecatching.settings import *
from io import BytesIO
import base64


def cal_area(vertices): #Gauss's area formula 高斯面积计算
    A = 0.0
    point_p = vertices[-1]
    for point in vertices:
        A += (point[1]*point_p[0] - point[0]*point_p[1])
        point_p = point
    return abs(A)/2

def cal_centroid(points):
    A = cal_area(points)
    c_x, c_y = 0.0, 0.0
    point_p = points[-1]
    for point in points:
        c_x +=((point[0] + point_p[0]) * (point[1]*point_p[0] - point_p[1]*point[0]))
        c_y +=((point[1] + point_p[1]) * (point[1]*point_p[0] - point_p[1]*point[0]))
        point_p = point

    return c_x / (6*A), c_y / (6*A)

def outlier(data, deviation):
    mean = np.mean(data)
    standard_deviation = np.std(data)
    distance_from_mean = abs(data - mean)
    max_deviations = deviation
    not_outlier = distance_from_mean < max_deviations * standard_deviation
    return not_outlier, mean, standard_deviation


def plot(user_input, pointPerPlot):
    with open('pupilOffsetXYList.pkl', 'rb') as f:
        res = pickle.load(f)
        f.close()
    pointPerPlot = int(pointPerPlot)
    res.time = pd.to_datetime(res.time)
    res = res[res.time > res.time[0]+datetime.timedelta(seconds=initTime)].reset_index(drop=True)

    xnot_outlier, xallmean, xallstd = outlier(res.x, 3)
    res = res[xnot_outlier]
    ynot_outlier, yallmean, yallstd = outlier(res.y, 3)
    res = res[ynot_outlier]

    xMax = max(list(res.x))
    xMin = min(list(res.x))
    yMax = max(list(res.y))
    yMin = min(list(res.y))

    res.x = [(j-xMin)/(xMax-xMin) for j in res.x]
    res.y = [(j-yMin)/(yMax-yMin) for j in res.y]

    timeDelta = (res.time[len(res.time)-1] - res.time[0]).seconds
    print(timeDelta)
    plotTimeDelta = int(timeDelta/plotNumber)
    print(plotTimeDelta)

    img = plt.imread("./static/%s"%user_input)
    plt.figure(figsize=(30, 25))
    xAll = []
    yAll = []

    for i in range(plotNumber):
        tStart = res.time[0] + datetime.timedelta(seconds=i*plotTimeDelta)
        tEnd = res.time[0] + datetime.timedelta(seconds=(i+1)*plotTimeDelta)
        dataTemp = res[(res.time >= tStart) & (res.time < tEnd)].dropna().reset_index(drop=True)
        xnot_outlier, xmean, xstd = outlier(dataTemp.x, 1)
        dataTemp = dataTemp[xnot_outlier]
        ynot_outlier, ymean, ystd = outlier(dataTemp.y, 1)
        dataTemp = dataTemp[ynot_outlier]

        # x_centroid, y_centroid = cal_centroid([[xTemp[j], yTemp[j]] for j in range(len(xTemp))])
        x = sample(list(dataTemp.x), pointPerPlot)
        y = sample(list(dataTemp.y), pointPerPlot)

        xAll.append(x)
        yAll.append(y)

        plt.subplot(plotNColumn, math.ceil(plotNumber / plotNColumn), i+1)
        if i == 0:
            plt.imshow(img, extent=[0, 1, 0, 1], alpha=1)
        else:
            plt.imshow(img, extent=[0, 1, 0, 1], alpha=0.5)
        # ax = plt.gca()

        # ellipse = Ellipse(xy=(xmean, ymean), width=xstd, height=ystd, alpha=0.5)
        # ax.add_patch(ellipse)
        #
        # ellipse = Ellipse(xy=(xmean, ymean), width=xstd*3, height=ystd*3, alpha=0.2)
        # ax.add_patch(ellipse)

        t = np.linspace(0, 2 * pi, 100)
        plt.fill(xmean + 1.5*xstd * np.cos(t), ymean + 1.5*ystd * np.sin(t), alpha=.5, color='#BEB8DC')
        # plt.fill(xmean + 3*xstd * np.cos(t), ymean + 3*ystd * np.sin(t), alpha=0.3, color='#E7DAD2')

        for j in range(i+1):
            if j != i:
                plt.scatter(xAll[j], yAll[j], s=200, alpha=0.1, label='%s'%j, color='blue')
            else:
                plt.scatter(xAll[j], yAll[j], s=200, alpha=1, label='%s' % j, color='blue')
            plt.legend()
        plt.title('From Time %s to Time %s, Initialization Time %s'%(i*plotTimeDelta, (i+1)*plotTimeDelta,initTime))
    # plt.savefig('./res%s.png'%user_input)
    plt.plot()
    sio = BytesIO()
    plt.savefig(sio, format='png', bbox_inches='tight', pad_inches=0.0)
    data = base64.encodebytes(sio.getvalue()).decode()
    plt.close()

    return data

# def pointsCalulator(data, pointPerPlot, plotNumber):
#
#     timeDelta = (data.time[len(data.time)-1] - data.time[0]).seconds
#     print(timeDelta)
#     plotTimeDelta = int(timeDelta/plotNumber)
#     print(plotTimeDelta)
#     xRaw = []
#     yRaw = []
#     x = []
#     y = []
#
#     for i in range(plotNumber):
#         tStart = data.time[0] + datetime.timedelta(seconds=i*plotTimeDelta+initTime)
#         tEnd = data.time[0] + datetime.timedelta(seconds=(i+1)*plotTimeDelta+initTime)
#         dataTemp = data[(data.time >= tStart) & (data.time < tEnd)].dropna().reset_index(drop=True)
#         dataTempSample = dataTemp.sample(n=pointPerPlot).reset_index(drop=True)
#         xRaw.append(dataTempSample.x)
#         yRaw.append(dataTempSample.y)
#
#     xMax = max([max(i) for i in xRaw])
#     xMin = min([min(i) for i in xRaw])
#     yMax = max([max(i) for i in yRaw])
#     yMin = min([min(i) for i in yRaw])
#
#     for i in range(plotNumber):
#         x.append([(j-xMin)/(xMax-xMin) for j in xRaw[i]])
#         y.append([(j-yMin) /(yMax-yMin) for j in yRaw[i]])
#
#     return x, y, plotTimeDelta

# def pointsAllCalulator(data, plotNumber, plotTimeDelta):
#
#     xRaw = []
#     yRaw = []
#     x = []
#     y = []
#
#     for i in range(plotNumber):
#         tStart = data.time[0] + datetime.timedelta(seconds=i*plotTimeDelta+initTime)
#         tEnd = data.time[0] + datetime.timedelta(seconds=(i+1)*plotTimeDelta+initTime)
#         dataTemp = data[(data.time >= tStart) & (data.time < tEnd)].dropna().reset_index(drop=True)
#         xRaw.append(dataTemp.x)
#         yRaw.append(dataTemp.y)
#
#     xMax = max([max(i) for i in xRaw])
#     xMin = min([min(i) for i in xRaw])
#     yMax = max([max(i) for i in yRaw])
#     yMin = min([min(i) for i in yRaw])
#
#     for i in range(plotNumber):
#         x.append([(j-xMin)/(xMax-xMin) for j in xRaw[i]])
#         y.append([(j-yMin) /(yMax-yMin) for j in yRaw[i]])
#
#     return x, y

# def plot(user_input):
#     with open('./result/pupilOffsetXYList.pkl', 'rb') as f:
#         res = pickle.load(f)
#         f.close()
#
#     for i in ['x', 'y']:
#         mean = np.mean(res[i])
#         print(mean)
#         standard_deviation = np.std(res[i])
#         print(standard_deviation)
#         distance_from_mean = abs(res[i] - mean)
#         max_deviations = 3
#         not_outlier = distance_from_mean < max_deviations * standard_deviation
#         res[i] = res[i][not_outlier]
#
#     res = res.reset_index(drop=True)
#     res.time = pd.to_datetime(res.time)
#
#     x, y, plotTimeDelta = pointsCalulator(res, pointPerPlot, plotNumber)
#
#     img = plt.imread("./static/%s"%user_input)
#     plt.figure(figsize=(30, 25))
#     for i in range(len(x)):
#         plt.subplot(plotNColumn, math.ceil(plotNumber / plotNColumn), i+1)
#         if i == 0:
#             plt.imshow(img, extent=[0, 1, 0, 1], alpha=1)
#         else:
#             plt.imshow(img, extent=[0, 1, 0, 1], alpha=0.5)
#         for j in range(i+1):
#             if j != i:
#                 plt.scatter(x[j], y[j], s=200, alpha=0.2, label = '%s'%j)
#             else:
#                 plt.scatter(x[j], y[j], s=200, alpha=1, label='%s' % j)
#             plt.legend()
#         plt.title('From Time %s to Time %s, Initialization Time %s'%(i*plotTimeDelta, (i+1)*plotTimeDelta,initTime))
#     plt.savefig('./result/res%s.png'%user_input)
#
#     sio = BytesIO()
#     plt.savefig(sio, format='png', bbox_inches='tight', pad_inches=0.0)
#     data = base64.encodebytes(sio.getvalue()).decode()
#     plt.close()
#
#     return data


# def plotall(user_input):
#
#     with open('./result/pupilOffsetXYList.pkl', 'rb') as f:
#         res = pickle.load(f)
#         f.close()
#
#     for i in ['x', 'y']:
#         mean = np.mean(res[i])
#         print(mean)
#         standard_deviation = np.std(res[i])
#         print(standard_deviation)
#         distance_from_mean = abs(res[i] - mean)
#         max_deviations = 3
#         not_outlier = distance_from_mean < max_deviations * standard_deviation
#         res[i] = res[i][not_outlier]
#
#     res = res.reset_index(drop=True)
#     res.time = pd.to_datetime(res.time)
#
#     x, y, plotTimeDelta = pointsCalulator(res, pointPerPlot, plotNumber)
#     x_all, y_all = pointsAllCalulator(res, plotNumber, plotTimeDelta)
#     img = plt.imread("./static/%s" % user_input)
#     plt.figure(figsize=(30, 25))
#     fig, ax = plt.subplots()
#     alpha_ratio = 1/len(x_all)
#     for i in range(len(x_all)):
#         ax.imshow(img, extent = [0, 1, 0, 1])
#         plt.plot(x_all[i], y_all[i], label = i, alpha = min(alpha_ratio*(i+1), 1), color = 'blue')
#         plt.legend()
#     plt.savefig('./result/res_all%s.png'%user_input)
#     sio = BytesIO()
#     plt.savefig(sio, format='png', bbox_inches='tight', pad_inches=0.0)
#     data = base64.encodebytes(sio.getvalue()).decode()
#     plt.close()
#
#     return data

if __name__ == "__main__":
    plot('Picture1.png', 5)


# -*- coding: utf-8 -*-
# Author: Jimei Shen
from setuptools import find_packages, setup

requires_list = open(f'EyeCatching/requirements.txt', 'r', encoding='utf8').readlines()
requires_list = [i.strip() for i in requires_list]

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='eyecatching',
    version='0.0.5',
    author="Jimei",
    author_email='shenjimei33@outlook.com',
    description="eye catching",
    python_requires="==3.7.*",
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_data={"EyeCatching/haarcascades": ["*"],
                  "EyeCatching/templates":["*"]},
    include_package_data=True,
    zip_safe=False,
    install_requires=requires_list,
    url="https://github.com/ShenJimei",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",

    ],
    entry_points={
        "console_scripts": [
            "realpython=eyecatching.main:main",
        ]}
)